import {
    VuexModule,
    Module,
    Action,
    Mutation,
    getModule
} from "vuex-module-decorators";
import { login, logout, getUserInfo } from "@/api/users";
import { getToken, setToken, removeToken } from "@/utils/cookies";
import store from "@/store";

export interface IUserState {
    token: string;
    name: string;
    username: string;
    userId: string;
    group: string;
    projectId: string;
    projects: any[];
    avatar: string;
    introduction: string;
    roles: string[];
}

@Module({ dynamic: true, store, name: "user" })
class User extends VuexModule implements IUserState {
    public token = getToken() || "";
    public name = "";
    public username = "yanruolan";
    public userId = "06328a53c69a41bb8f5bb1a552c6e8d6";
    public group = "WD";// 集团
    public projectId = "Pj4403070003";// 深圳龙岗万达广场
    public projects: any[] = [{
        id: "Pj4403070003",
        name: "深圳龙岗万达广场"
    }];// 用户项目列表
    public avatar = "";
    public introduction = "";
    public roles: string[] = [];

    @Mutation
    private SET_TOKEN(token: string) {
        this.token = token;
    }

    @Mutation
    private SET_NAME(name: string) {
        this.name = name;
    }

    @Mutation
    private SET_GROUP(group: string) {
        this.group = group;
    }

    @Mutation
    private SET_PROJECTID(projectId: string) {
        this.projectId = projectId;
    }

    @Mutation
    private SET_PROJECTS(projects: any[]) {
        this.projects = projects;
    }

    @Mutation
    private SET_AVATAR(avatar: string) {
        this.avatar = avatar;
    }

    @Mutation
    private SET_INTRODUCTION(introduction: string) {
        this.introduction = introduction;
    }

    @Mutation
    private SET_ROLES(roles: string[]) {
        this.roles = roles;
    }

    @Action
    public async Login(userInfo: { username: string; password: string }) {
        let { username, password } = userInfo;
        username = username.trim();
        const { data } = await login({ username, password });
        setToken(data.accessToken);
        this.SET_TOKEN(data.accessToken);
    }

    @Action
    public ResetToken() {
        removeToken();
        this.SET_TOKEN("");
        this.SET_ROLES([]);
    }

    @Action
    public async GetUserInfo() {
        if (this.token === "") {
            throw Error("GetUserInfo: token is undefined!");
        }
        const { data } = await getUserInfo({
            /* Your params here */
        });
        if (!data) {
            throw Error("Verification failed, please Login again.");
        }
        const { roles, name, avatar, introduction } = data.user;
        // roles must be a non-empty array
        if (!roles || roles.length <= 0) {
            throw Error("GetUserInfo: roles must be a non-null array!");
        }
        this.SET_ROLES(roles);
        this.SET_NAME(name);
        this.SET_AVATAR(avatar);
        this.SET_INTRODUCTION(introduction);
    }

    @Action
    public async LogOut() {
        if (this.token === "") {
            throw Error("LogOut: token is undefined!");
        }
        await logout();
        removeToken();
        this.SET_TOKEN("");
        this.SET_ROLES([]);
    }
}

export const UserModule = getModule(User);
